#!/usr/bin/env python3

import plot_eddy_contours
from matplotlib import animation


def bbox_union(b1, b2):
    if b1 is None:
        return b2
    elif b2 is None:
        return b1
    else:
        return [
            min(b1[0], b2[0]),
            min(b1[1], b2[1]),
            max(b1[2], b2[2]),
            max(b1[3], b2[3]),
        ]


def compute_bbox(ishape_list, reader_outer):
    """ishape_list should be a list."""

    bbox = None

    for ishape in ishape_list:
        try:
            bbox = bbox_union(bbox, reader_outer.shape(ishape).bbox)
        except AttributeError:
            pass

    return bbox


def func(d, ax, i_slices, ishape_lists_all, SHPC, bbox, light):
    """To be passed as argument to the animation
    function. ishape_lists_all is a dictionary.

    """

    ax.cla()
    ax.set_xlim(bbox[0], bbox[2])
    ax.set_ylim(bbox[1], bbox[3])

    for orientation in ["Anticyclones", "Cyclones"]:
        plot_eddy_contours.snapshot(
            ax,
            ishape_lists_all[orientation][d],
            SHPC,
            i_slices[d],
            orientation,
            light=light,
        )

    ax.gridlines(draw_labels=True)
    ax.coastlines()
    ax.set_title(f"d = {d}", y=1.05)


def anim_eddies(fig, ax, SHPC, window, d_min, d_max, light):
    """SHPC is an instance of SHPC_class. We define i_slices and
    ishape_lists_all[orientation] here rather than in func because we
    need them to compute bbox.

    """

    ishape_lists_all = {}
    i_slices = {}

    for orientation in ["Anticyclones", "Cyclones"]:
        ishape_lists_all[orientation] = {}

        for d in range(d_min, d_max + 1):
            (
                i_slices[d],
                ishape_lists_all[orientation][d],
            ) = plot_eddy_contours.select_ishapes(d, SHPC, orientation, window)

    if window is None:
        bbox = None

        for orientation in ["Anticyclones", "Cyclones"]:
            for d in range(d_min, d_max + 1):
                b2 = compute_bbox(
                    ishape_lists_all[orientation][d],
                    SHPC.get_reader(
                        i_slices[d], orientation, "outermost_contour"
                    ),
                )
                bbox = bbox_union(bbox, b2)
    else:
        bbox = window

    ani = animation.FuncAnimation(
        fig,
        func,
        range(d_min, d_max + 1),
        fargs=(ax, i_slices, ishape_lists_all, SHPC, bbox, light),
        interval=500,
    )
    return ani


if __name__ == "__main__":
    import argparse
    import sys
    import matplotlib.pyplot as plt
    import cartopy.crs as ccrs
    import pygifsicle
    from os import path
    import util_eddies

    # Process arguments:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "shpc_dir", help="directory containing the " "collection of shapefiles"
    )
    parser.add_argument(
        "-d", "--dates", type=int, nargs=2, metavar=("d_min", "d_max")
    )
    parser.add_argument(
        "-l", "--light", help="lighter plot", action="store_true"
    )
    parser.add_argument(
        "-w",
        "--window",
        help="choose a limited plot window",
        type=float,
        nargs=4,
        metavar=("llcrnrlon", "llcrnrlat", "urcrnrlon", "urcrnrlat"),
    )
    args = parser.parse_args()
    # --

    SHPC = util_eddies.SHPC_class(args.shpc_dir, def_orient="Anticyclones")

    # Do some checks on first orientation:

    if args.dates:
        d_min = args.dates[0]
        d_max = args.dates[1]

        if (
            not SHPC.d_init[0]
            <= d_min
            < d_max
                <= SHPC.d_init[-1] + SHPC.n_dates[SHPC.n_slices - 1] - 1
        ):
            sys.exit("Bad dates specified")
    else:
        d_min = SHPC.d_init[0]
        d_max = SHPC.d_init[0] + SHPC.n_dates[0] - 1
    # --

    fig = plt.figure()
    projection = ccrs.PlateCarree()
    ax = plt.axes(projection=projection)
    ani = anim_eddies(fig, ax, SHPC, args.window, d_min, d_max, args.light)
    ani.save("eddy_contours.gif", writer="imagemagick")

    pygifsicle.gifsicle("eddy_contours.gif", options=["--no-loopcount"])
    # (The repeat = False option of the save method of
    # FuncAnimation does not work.)

    print('Created file "eddy_contours.gif".')
