program test_get_1_outerm

  ! This program targets an extremum identified by its orientation and
  ! its coordinates in projection space. It reads a field of SSH. It
  ! writes an SHPC containing a single record, and shapefiles
  ! cont_list and cont_list_proj.

  ! Libraries:
  use contour_531, only: polyline
  use jumble, only: csvread, rad_to_deg, get_command_arg_dyn
  use shapelib, only: shpfileobject, ftdouble, shpt_polygon, shpclose
  use shapelib_03, only: shp_create_03, dbf_add_field_03, &
       shp_append_object_03, dbf_write_attribute_03

  use config_m, only: config
  use derived_types, only: eddy, shpc_slice_handler, null_ssh_contour, &
       missing_speed, ssh_contour
  use get_1_outerm_m, only: get_1_outerm
  use input_ssh_m, only: input_ssh, uniform_lon_lat
  use invert_proj_m, only: invert_proj
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use write_eddy_m, only: write_eddy

  implicit none

  integer:: date = 0, i, ifield, ishape

  real, allocatable:: ssh(:, :)
  ! (1 - max_radius(1):nlon + max_radius(1), nlat) if the grid is periodic
  ! in longitude, else (nlon, nlat). Sea-surface height, in m.

  integer:: ind_targ_extr(2) = [19, 11]
  ! indices in the two dimensions of target extremum

  real:: innermost_level = 0.2933
  ! level of innermost contour, for each extremum

  logical:: cyclone = .true.
  logical periodic ! grid is periodic in longitude
  TYPE(shpc_slice_handler) hshpc
  type(eddy) e

  integer, allocatable:: outside_points(:, :) ! (2, :)
  ! coordinates in projection space of all the significant extrema,
  ! except the target extremum

  integer, parameter:: n_max_cont = 31

  type(polyline) cont_list(n_max_cont)
  ! Contour list, with points given by longitude and latitude, in
  ! radians. Defined only for subscripts 1:n_cont. If n_cont >= 1, the
  ! outermost contour is element with subscript n_cont. For i <=
  ! n_cont, the order of points in cont_list(i) (clockwise or
  ! counter-clockwise) is not specified.

  type(polyline) cont_list_proj(n_max_cont)
  ! Contour list, with points given by projection coordinates. Defined
  ! only for subscripts 1:n_cont. If n_cont >= 1, the outermost
  ! contour is element with subscript n_cont. The contours are in
  ! monotonic order of ssh. The contour with a given subscript in
  ! cont_list_proj corresponds to the contour with the same subscript
  ! in cont_list. For i <= n_cont, the order of points in
  ! cont_list_proj(i) (clockwise or counter-clockwise) is not
  ! specified.

  real cont_list_ssh(n_max_cont)
  integer n_cont ! number of good contours found and stored in cont_list
  TYPE(shpfileobject) hshp
  namelist /main_nml/ ind_targ_extr, innermost_level, cyclone, date
  logical exist
  character(len = :), allocatable:: ssh_fname, uv_fname ! file names

  !----------------------------------------------------------------

  write(unit = *, nml = main_nml)
  print  *, "test_get_1_outerm: Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)
  call config(leave_trace = .true.)
  call get_command_arg_dyn(1, ssh_fname, "Required arguments: h-file uv-file")
  call get_command_arg_dyn(2, uv_fname, "Required arguments: h-file uv-file")
  call input_ssh(periodic, ssh, ssh_fname, uv_fname, uv_fname, &
       leave_trace = .true.)
  inquire(file = "outside_points.csv", exist = exist)

  if (exist) then
     call csvread("outside_points.csv", outside_points, first_r = 2)
     outside_points = transpose(outside_points)
  else
     print *, '"outside_points.csv" not found. Assuming no outside points...'
     allocate(outside_points(2, 0))
  end if

  e%extr%coord_proj = ind_targ_extr
  e%extr%coord = invert_proj(ind_targ_extr)
  call get_1_outerm(e%out_cont, cont_list, cont_list_proj, cont_list_ssh, &
       n_cont, cyclone, real(ind_targ_extr), innermost_level, &
       real(outside_points), ssh, corner = [1., 1.])

  if (e%out_cont%closed) then
     e%extr%ssh = ssh(ind_targ_extr(1), ind_targ_extr(2))
     e%cyclone = cyclone
     e%speed_cont = null_ssh_contour()
     e%max_speed = missing_speed
     call shpc_create(hshpc, shpc_dir = "SHPC", cyclone = cyclone, slice = 0, &
          with_proj = .not. uniform_lon_lat)
     call write_eddy(e, hshpc, date, i = 1)
     write(hshpc%unit, fmt = *) 0
     CALL shpc_close(hshpc)
     print *, 'test_get_1_outerm: Created "SHPC".'

     ! cont_list shapefile:

     call shp_create_03("cont_list", shpt_polygon, hshp)
     call dbf_add_field_03(ifield, hshp, 'speed', ftdouble, nwidth = 13, &
          ndecimals = 6)

     do i = 1, n_cont
        call shp_append_object_03(ishape, hshp, shpt_polygon, &
             cont_list(i)%points * rad_to_deg)
        call dbf_write_attribute_03(hshp, ishape, ifield, missing_speed)
     end do

     CALL shpclose(hshp)
     print *, 'test_get_1_outerm: Created shapefile "cont_list".'

     ! cont_list_proj shapefile:

     call shp_create_03("cont_list_proj", shpt_polygon, hshp)
     call dbf_add_field_03(ifield, hshp, 'ssh', ftdouble, nwidth = 13, &
          ndecimals = 6)

     do i = 1, n_cont
        call shp_append_object_03(ishape, hshp, shpt_polygon, &
             cont_list_proj(i)%points)
        call dbf_write_attribute_03(hshp, ishape, ifield, cont_list_ssh(i))
     end do

     CALL shpclose(hshp)
     print *, 'test_get_1_outerm: Created shapefile "cont_list_proj".'
  else
     print *, "test_get_1_outerm: Could not find an outermost contour."
  end if

end program test_get_1_outerm
