#!/usr/bin/env python3

"""This script reads an edgelist and extracts a component using
networkx. The component is written to a new edgelist file.

Note that the script loads the whole graph into memory. For
the global graph on 30 years, this requires about 13 GiB of main
memory.

The file e_overestim.txt has to be in the same directory as the
edgelist file given as argument of the script.

"""

import networkx as nx

def add_component(G: nx.DiGraph, source: int):
    """Add the weakly-connected component containing node source.

    The component is a set of nodes. Note this is not the same as the
    union of source, its descendants and ancestors.

    """

    if "component" in G.nodes[source]:
        print(f"add_component: component containing {source} already added")
    else:
        undirected_G = nx.to_undirected(G)
        node_generator = nx.dfs_preorder_nodes(undirected_G, source)
        component = set(node_generator)

        if "component_list" in G.graph:
            G.graph["component_list"].append(component)
        else:
            G.graph["component_list"] = [component]

        for n in component:
            G.nodes[n]["component"] = component

if __name__ == "__main__":
    import sys

    import report_graph

    if len(sys.argv) != 4:
        sys.exit("Required arguments: input-file node-number output-file")

    node = int(sys.argv[2])
    G = report_graph.read_eddy_graph(sys.argv[1])
    add_component(G, node)
    H = G.subgraph(G.nodes[node]["component"])
    len_H = len(H)
    print("Number of nodes:", len_H)
    print("Number of edges:", H.number_of_edges())
    nx.write_edgelist(H, sys.argv[3], data = False)
