#!/usr/bin/env python3

"""This script reads an edgelist and prints some information about the
corresponding graph. Note that the script loads the whole graph into
memory, with networkx. For the global graph on 30 years, this requires
about 13 GiB of main memory. The file `e_overestim.txt` has to be in
the same directory as the edgelist file given as argument of the
script.

"""

from os import path
import os

import networkx as nx

import util_eddies


def read_eddy_graph(edgelist, shpc_dir=None, orientation="Cyclones"):
    if os.access(edgelist, os.R_OK):
        G = nx.read_edgelist(edgelist, create_using=nx.DiGraph, nodetype=int)
        dir_edgelist = path.dirname(edgelist)
        e_overestim_file = path.join(dir_edgelist, "e_overestim.txt")

        with open(e_overestim_file) as f_obj:
            G.graph["e_overestim"] = int(f_obj.read())

        if shpc_dir is not None:
            # Read and set attributes of eddies:
            SHPC = util_eddies.SHPC_class(shpc_dir, orientation)
            set_attribute(G, SHPC, orientation)
    else:
        print(edgelist, "not readable")
        G = nx.DiGraph()

    return G


def set_attribute(G, SHPC, orientation):
    if (
        G.number_of_nodes()
        <= SHPC.get_n_shapes(0, orientation) * SHPC.n_slices / 100
    ):
        for n in G:
            i_slice, ishape = SHPC.comp_ishape_n(
                n, G.graph["e_overestim"], orientation
            )
            reader = SHPC.get_reader(i_slice, orientation, "extremum")
            shape_rec = reader.shapeRecord(ishape)
            G.add_node(
                n,
                coordinates=tuple(shape_rec.shape.points[0]),
                speed=shape_rec.record.speed,
                ssh=shape_rec.record.ssh,
            )
    else:
        for i_slice in range(SHPC.n_slices):
            reader = SHPC.get_reader(i_slice, orientation, "extremum")

            for shape_rec in reader:
                n = util_eddies.date_eddy_to_node(
                    shape_rec.record.date,
                    shape_rec.record.eddy_index,
                    G.graph["e_overestim"],
                )

                if n in G:
                    G.add_node(
                        n,
                        coordinates=tuple(shape_rec.shape.points[0]),
                        speed=shape_rec.record.speed,
                        ssh=shape_rec.record.ssh,
                    )


if __name__ == "__main__":
    import argparse

    # Parse arguments:
    parser = argparse.ArgumentParser()
    parser.add_argument("edgelist_file")
    parser.add_argument(
        "-m",
        "--merging",
        help="Report merging and splitting events",
        action="store_true",
    )
    args = parser.parse_args()

    G = read_eddy_graph(args.edgelist_file)
    len_G = len(G)
    print("Number of nodes:", len_G)
    print(
        "(This should be the number of eddies minus the number of isolated "
        "nodes.)"
    )
    print("Number of edges:", G.number_of_edges())
    print(
        "Number of weakly connected components:",
        nx.number_weakly_connected_components(G),
    )
    n1 = len_G
    n2 = len_G

    if len_G >= 1:
        # First and last dates:
        node_index = min(G.nodes)
        my_date = util_eddies.node_to_date_eddy(
            node_index, G.graph["e_overestim"], only_date=True
        )
        print("First date:", my_date)
        node_index = max(G.nodes)
        my_date = util_eddies.node_to_date_eddy(
            node_index, G.graph["e_overestim"], only_date=True
        )
        print("Last date:", my_date)

    if args.merging:
        print("\nMerging:")

    for n, d in G.in_degree():
        if d >= 1:
            n2 -= 1
            if args.merging and d >= 2:
                print(list(G.predecessors(n)), "->", n)

    if args.merging:
        print("---")
        print("Splitting:")

    for n, d in G.out_degree():
        if d >= 1:
            n1 -= 1
            if args.merging and d >= 2:
                print(n, "->", list(G.successors(n)))

    if args.merging:
        print("---")

    print("Number of leaves:", n1)
    print("Number of roots:", n2)
