module overlap_m

  implicit none

contains

  subroutine overlap(flow, e_overestim, k, delta, j)

    ! This procedure finds edges between flow(j - delta) and flow(j),
    ! corresponding to date indices k - delta and k. It writes these
    ! edges. It updates flow(j - delta)%list%delta_out and
    ! flow(j)%list%delta_in.

    ! Libraries:
    use contour_531, only: polyline

    use copy_shift_m, only: copy_shift
    use candidate_overlap_m, only: candidate_overlap
    use config_graph_m, only: dist_lim, min_inters_speed, min_inters_out
    use derived_types, only: snapshot
    use inters_lines_m, only: inters_lines
    use read_grid_m, only: nlon, nlat, periodic
    use unit_edge_m, only: unit_edge

    type(snapshot), intent(inout):: flow(:) ! (max_delta + 1)

    integer, intent(in):: e_overestim
    ! over-estimation of the number of eddies at each date

    integer, intent(in):: k ! date index
    integer, intent(in):: delta ! between 1 and max_delta

    integer, intent(in):: j
    ! position in time window, between 2 and max_delta + 1

    ! Local:

    integer i1 ! eddy index of predecessor
    integer i2 ! eddy index of successor
    integer l, n_select
    type(polyline) p_shifted
    logical assoc_eddies ! association between eddies
    real delta_long ! difference in longitude, in rad

    integer, allocatable:: selection(:)
    ! identifying numbers of a selection of extrema

    ! Window around each extremum:
    integer llc(2) ! indices in global grid of lower left corner
    integer urc(2) ! indices in global grid of upper right corner

    !-----------------------------------------------------------------------

    loop_i1: do i1 = 1, flow(j - delta)%number_extr
       associate (e1 => flow(j - delta)%list(i1))
         ! Define the geographical window around each eddy extremum:

         llc = e1%extr%coord_proj - dist_lim
         urc = e1%extr%coord_proj + dist_lim

         llc(2) = max(llc(2), 1)
         urc(2) = min(urc(2), nlat)

         if (.not. periodic) then
            llc(1) = max(llc(1), 1)
            urc(1) = min(urc(1), nlon)
         end if

         ! Pre-select potential successors:
         selection = candidate_overlap(flow(j)%extr_map(llc(1):urc(1), &
              llc(2):urc(2)), flow(j)%list, e1%delta_out, delta)

         n_select = size(selection)

         DO l = 1, n_select
            i2 = selection(l)
            associate (e2 => flow(j)%list(i2))
              ! Assertion: {e1%delta_out >= delta .or. e2%delta_in >=
              ! delta}
              delta_long = e1%extr%coord(1) - e2%extr%coord(1)

              if (e1%speed_cont%n_points == 0 &
                   .and. e2%speed_cont%n_points /= 0) then
                 p_shifted = copy_shift(e1%out_cont%polyline, - delta_long)
                 assoc_eddies = inters_lines(p_shifted, &
                      e2%speed_cont%polyline, min_inters_speed)
                 if (.not. assoc_eddies) assoc_eddies &
                      = inters_lines(p_shifted, e2%out_cont%polyline, &
                      min_inters_out)
              elseif (e1%speed_cont%n_points /= 0 &
                   .and. e2%speed_cont%n_points == 0) then
                 p_shifted = copy_shift(e2%out_cont%polyline, delta_long)
                 assoc_eddies = inters_lines(e1%speed_cont%polyline, &
                      p_shifted, min_inters_speed)
                 if (.not. assoc_eddies) assoc_eddies &
                      = inters_lines(e1%out_cont%polyline, p_shifted, &
                      min_inters_out)
              elseif (e1%speed_cont%n_points == 0 &
                   .and. e2%speed_cont%n_points == 0) then
                 p_shifted = copy_shift(e2%out_cont%polyline, delta_long)
                 assoc_eddies = inters_lines(e1%out_cont%polyline, p_shifted, &
                      min(min_inters_speed, min_inters_out))
              else
                 if (min_inters_speed <= 1.) then
                    p_shifted = copy_shift(e2%speed_cont%polyline, delta_long)
                    assoc_eddies = inters_lines(e1%speed_cont%polyline, &
                         p_shifted, min_inters_speed)
                 else
                    assoc_eddies = .false.
                 end if

                 if (.not. assoc_eddies .and. min_inters_out <= 1.) then
                    p_shifted = copy_shift(e2%out_cont%polyline, delta_long)
                    assoc_eddies = inters_lines(e1%out_cont%polyline, &
                         p_shifted, min_inters_out)
                 end if
              end if

              if (assoc_eddies) then
                 write(unit_edge, fmt = *) (k - delta) * e_overestim + i1, &
                      k * e_overestim + i2
                 e1%delta_out = min(e1%delta_out, delta)
                 e2%delta_in = min(e2%delta_in, delta)
              end if
            end associate
         end DO
       end associate
    end do loop_i1

  end subroutine overlap

end module overlap_m
